@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2009,2018-2020 Free Software Foundation, Inc.              */
+/*  Copyright (C) 2009,2018-2021 Free Software Foundation, Inc.              */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -491,12 +491,10 @@ bit_read_RD (Bit_Chain *dat)
   unsigned char byte[8];
   double *result;
 
+  //CHK_OVERFLOW_PLUS (8, __FUNCTION__, bit_nan ())
   // TODO: big-endian
   for (i = 0; i < 8; i++)
-    {
-      byte[i] = bit_read_RC (dat);
-      CHK_OVERFLOW(__FUNCTION__, bit_nan ())
-    }
+    byte[i] = bit_read_RC (dat);
 
   result = (double *)byte;
   return (*result);
@@ -1118,6 +1116,12 @@ bit_read_DD (Bit_Chain *dat, double default_value)
     }
 }
 
+int
+bit_eq_DD (double value, double default_value)
+{
+  return fabs (value - default_value) < 1e-12;
+}
+
 /** Write bit-double with default.
  */
 BITCODE_BB
@@ -1422,6 +1426,7 @@ bit_read_fixed (Bit_Chain *restrict dat, BITCODE_RC *restrict dest,
       loglevel = dat->opts & DWG_OPTS_LOGLEVEL;
       LOG_ERROR ("%s buffer overflow at pos %lu, size %lu", __FUNCTION__,
                  dat->byte, dat->size)
+      memset (dest, 0, length);
       return;
     }
   if (dat->bit == 0)
@@ -1504,6 +1509,17 @@ bit_read_bits (Bit_Chain *dat, unsigned long bits)
 void
 bit_write_TF (Bit_Chain *restrict dat, BITCODE_TF restrict chain, unsigned int length)
 {
+  if (!chain)
+    {
+      loglevel = dat->opts & DWG_OPTS_LOGLEVEL;
+      LOG_ERROR ("Empty TF with length %u", length);
+      if (length <= 128) // either chain or length is wrong
+        {
+          for (unsigned int i = 0; i < length; i++)
+            bit_write_RC (dat, 0);
+        }
+      return;
+    }
   if (dat->bit == 0 && dat->byte + length < dat->size)
     {
       memcpy (&dat->chain[dat->byte], chain, length);
@@ -1537,9 +1553,14 @@ bit_read_TV (Bit_Chain *restrict dat)
     }
   for (i = 0; i < length; i++)
     chain[i] = bit_read_RC (dat);
-  // normally not needed, as the DWG itself contains the ending \0 as last char
+  // check if the string is already zero-terminated or not.
+  // only observed >=r2004 as writer app
+  if (length > 0 && dat->from_version > R_2000 && chain[length - 1] != '\0')
+    LOG_HANDLE ("TV-not-ZERO %u\n ", length)
+  else if (length > 0 && dat->from_version <= R_2000 && chain[length - 1] == '\0')
+    LOG_HANDLE ("TV-ZERO %u\n", length)
+  // normally not needed, as the DWG since r2004 itself contains the ending \0 as last char
   chain[i] = '\0';
-
   return (char *)chain;
 }
 
@@ -1575,7 +1596,7 @@ bit_embed_TU_size (BITCODE_TU restrict wstr, const int len)
       if ((uintptr_t)wstr % SIZEOF_SIZE_T)
         {
           unsigned char *b = (unsigned char *)wstr;
-          c = (b[1] << 8) + b[0];
+          c = TU_to_int(b);
           wstr++;
         }
       else
@@ -1627,14 +1648,14 @@ bit_wcs2nlen (const BITCODE_TU restrict wstr, const size_t maxlen)
   if ((uintptr_t)wstr % SIZEOF_SIZE_T)
     {
       unsigned char *b = (unsigned char *)wstr;
-      uint16_t c = (b[0] << 8) + b[1];
+      uint16_t c = TU_to_int(b);
       while (c)
         {
           len++;
           if (len >= maxlen)
             return 0;
           b += 2;
-          c = (b[0] << 8) + b[1];
+          c = TU_to_int(b);
         }
       return len;
     }
@@ -1670,12 +1691,12 @@ bit_wcs2len (const BITCODE_TU restrict wstr)
   if ((uintptr_t)wstr % SIZEOF_SIZE_T)
     {
       unsigned char *b = (unsigned char *)wstr;
-      uint16_t c = (b[0] << 8) + b[1];
+      uint16_t c = TU_to_int(b);
       while (c)
         {
           len++;
           b += 2;
-          c = (b[0] << 8) + b[1];
+          c = TU_to_int(b);
         }
       return len;
     }
@@ -1705,11 +1726,11 @@ bit_wcs2cpy (BITCODE_TU restrict dest, const BITCODE_TU restrict src)
   if ((uintptr_t)src % SIZEOF_SIZE_T)
     {
       unsigned char *b = (unsigned char *)src;
-      *d = (b[0] << 8) + b[1];
+      *d = TU_to_int(b);
       while (*d)
         {
           b += 2;
-          *d = (b[0] << 8) + b[1];
+          *d = TU_to_int(b);
         }
       return dest;
     }
@@ -1739,11 +1760,11 @@ bit_wcs2cmp (BITCODE_TU restrict dest, const BITCODE_TU restrict src)
   if ((uintptr_t)src % SIZEOF_SIZE_T)
     {
       unsigned char *s = (unsigned char *)src;
-      uint16_t s1 = (s[0] << 8) + s[1];
+      uint16_t s1 = TU_to_int(s);
       while (*d++ == s1)
         {
           s += 2;
-          s1 = (s[0] << 8) + s[1];
+          s1 = TU_to_int(s);
         }
       return (*d || *s1) ? 1 : 0;
     }
@@ -1798,12 +1819,16 @@ bit_embed_TU (BITCODE_TU restrict wstr)
 }
 
 /** Write ASCIIZ text.
+    Starting with r2004 as writer (not target version) acad always
+    writes a terminating zero.
  */
 void
 bit_write_TV (Bit_Chain *restrict dat, BITCODE_TV restrict chain)
 {
   int i;
-  int length = (chain && *chain) ? strlen ((const char *)chain) + 1 : 0;
+  int length = (chain && *chain) ? strlen ((const char *)chain) : 0;
+  if (dat->version <= R_2000 && length)
+    length++;
   bit_write_BS (dat, length);
   for (i = 0; i < length; i++)
     bit_write_RC (dat, (unsigned char)chain[i]);
@@ -1822,7 +1847,7 @@ bit_write_T (Bit_Chain *restrict dat, BITCODE_T restrict s)
 {
   int i, length;
 
-  // only if from r2007+ DWG, not JSON, DXF, add API
+  // only if from r2007+ DWG. not JSON, DXF, add API.
   if (IS_FROM_TU (dat))
     {
       // downconvert TU to TV
@@ -2233,7 +2258,7 @@ bit_convert_TU (const BITCODE_TU restrict wstr)
   if ((uintptr_t)wstr % SIZEOF_SIZE_T)
     {
       unsigned char *b = (unsigned char*)wstr;
-      c = (b[1] << 8) + b[0];
+      c = TU_to_int(b);
       while (c)
         {
           len++;
@@ -2244,7 +2269,7 @@ bit_convert_TU (const BITCODE_TU restrict wstr)
                 len++;
             }
           b += 2;
-          c = (b[1] << 8) + b[0];
+          c = TU_to_int(b);
         }
     }
   else
@@ -2277,7 +2302,7 @@ bit_convert_TU (const BITCODE_TU restrict wstr)
     {
       unsigned char *b = (unsigned char*)wstr;
       // possible gcc bug
-      c = (b[1] << 8) + b[0];
+      c = TU_to_int(b);
       while (c && i < len)
         {
           if (c < 0x80)
@@ -2297,7 +2322,7 @@ bit_convert_TU (const BITCODE_TU restrict wstr)
             }
 
           b += 2;
-          c = (b[1] << 8) + b[0];
+          c = TU_to_int(b);
         }
     }
   else
@@ -2370,7 +2395,7 @@ bit_TU_to_utf8_len (const BITCODE_TU restrict wstr, const int len)
     {
       unsigned char *b = (unsigned char*)wstr;
       // possible gcc bug
-      c = (b[1] << 8) + b[0];
+      c = TU_to_int(b);
       while (c && i < len)
         {
           if (c < 0x80)
@@ -2392,7 +2417,7 @@ bit_TU_to_utf8_len (const BITCODE_TU restrict wstr, const int len)
             }
 
           b += 2;
-          c = (b[1] << 8) + b[0];
+          c = TU_to_int(b);
         }
     }
   else
@@ -2433,8 +2458,8 @@ bit_TU_to_utf8_len (const BITCODE_TU restrict wstr, const int len)
         HANDLER (OUTPUT, "ERROR: overlarge unicode codepoint U+%0X", c);
      */
     }
-  if (i <= len + 1)
-    str[i] = '\0';
+  EXTEND_SIZE(str, i + 1, len);
+  str[i] = '\0';
   return str;
 }
 
@@ -2445,20 +2470,19 @@ bit_TU_to_utf8_len (const BITCODE_TU restrict wstr, const int len)
 */
 char *
 bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
-                const int destlen, const unsigned cquoted)
+                const int destlen, const int srclen, const unsigned cquoted)
 {
   unsigned char c;
   unsigned char *s = (unsigned char *)src;
   const char* endp = dest + destlen;
+  const unsigned char* ends = src + srclen;
   char *d = dest;
 
   while ((c = *s++))
     {
       if (dest >= endp)
-        {
-          return NULL;
-        }
-      else if (cquoted && c == '\\' && dest+1 < endp &&
+        return NULL;
+      else if (cquoted && c == '\\' && dest+1 < endp && s + 1 <= ends &&
           // skip \" to " and \\ to \.
           (*s == '"' || *s == '\\' || *s == 'r' || *s == 'n'))
         {
@@ -2474,7 +2498,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
               }
         }
       // \uxxxx => \U+XXXX as in bit_embed_TU
-      else if (c == '\\' && dest+7 < endp && *s == 'u')
+      else if (c == '\\' && dest+7 < endp && *s == 'u' && s + 5 <= ends)
         {
           *dest++ = c;
           *dest++ = 'U';
@@ -2489,7 +2513,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
         {
           *dest++ = c;
         }
-      else if ((c & 0xe0) == 0xc0)
+      else if ((c & 0xe0) == 0xc0 && s + 1 <= ends)
         {
           /* ignore invalid utf8 for now */
           if (dest+7 < endp)
@@ -2510,7 +2534,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
       else if ((c & 0xf0) == 0xe0)
         {
           /* warn on invalid utf8 */
-          if (dest+2 < endp &&
+          if (dest+2 < endp && s + 1 <= ends &&
               (*s < 0x80 || *s > 0xBF || *(s+1) < 0x80 || *(s+1) > 0xBF))
             {
               LOG_WARN ("utf-8: BAD_CONTINUATION_BYTE %s", s);
@@ -2519,7 +2543,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
             {
               LOG_WARN ("utf-8: NON_SHORTEST %s", s);
             }
-          if (dest+7 < endp)
+          if (dest+7 < endp && s + 1 <= ends)
             {
               BITCODE_RS wc = ((c & 0x0f) << 12) | ((*s & 0x3f) << 6) | (*(s+1) & 0x3f);
               *dest++ = '\\';
@@ -2532,9 +2556,13 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
             }
           else
             return NULL;
+          if (s + 2 > ends)
+            break;
           s++;
           s++;
         }
+      if (s >= ends)
+        break;
       /* everything above 0xf0 exceeds ucs-2, 4-6 byte seqs */
     }
 
@@ -2619,8 +2647,12 @@ bit_eq_T (Bit_Chain *restrict dat, const BITCODE_T restrict wstr1, const char *r
 int
 bit_eq_TU (const char *restrict str, BITCODE_TU restrict wstr)
 {
-  char *utf8 = bit_convert_TU (wstr);
-  int result = strcmp (str, utf8) ? 0 : 1;
+  char *utf8;
+  int result;
+  if (!str)
+    return (wstr && *wstr) ? 0 : 1;
+  utf8 = bit_convert_TU (wstr);
+  result = utf8 ? (strcmp (str, utf8) ? 0 : 1) : 0;
   free (utf8);
   return result;
 }
@@ -2640,7 +2672,7 @@ int bit_empty_T (Bit_Chain *restrict dat, BITCODE_T restrict str)
       if ((uintptr_t)str % SIZEOF_SIZE_T)
         {
           unsigned char *b = (unsigned char*)str;
-          c = (b[0] << 8) + b[1];
+          c = TU_to_int(b);
           return !c;
         }
 #endif
